/*
 * Log4JFugue - Application Sonification
 * Copyright (C) 2008-2009  Brian Tarbox
 *
 * http://www.log4jfugue.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

package org.log4jfugue;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Properties;

import org.apache.log4j.Logger;

/**
 * This is the central repository of knowledge about the system.
 * It contains the accumulators of located search string counts, which are
 * used by both the message receiver and the sound builder classes
 * @author btarbox
 *
 */
public class Music {
	Logger log = Logger.getLogger(Music.class);
	public static final int MAX_MIDI_VOICES = 16;
	protected int counts[][] = new int[2][MAX_MIDI_VOICES];
	protected Integer buildIndex;
	private InstrumentVoice[] instruments = new InstrumentVoice[MAX_MIDI_VOICES];
	private HashMap<String, Integer> instrumentIndexes = new HashMap<String, Integer>();
	
	public Music() {
		buildIndex = 0;
	}

	/**
	 * Used by Spring DI this accepts the set of properties that define the
	 * search strings and associated instruments
	 * @param messages
	 */
	public void setProperties(Properties messages) {
    	Enumeration e = messages.propertyNames();
    	int index = 0;
    	while(e.hasMoreElements()) {
    		String key = (String)e.nextElement();
    		String value = messages.getProperty(key);
    		InstrumentVoice thisVoice = new InstrumentVoice(index, value, key);
    		instruments[index] = thisVoice;
    		log.info("added voice " + index + " " + key + " " + value + " to music array");
    		instrumentIndexes.put(key, index);  // allow quick lookup of index by name
    		index++;
    	}
	}
	
	public int getInstrumentIndex(String name){
		return(instrumentIndexes.get(name));
	}
	
	/**
	 * Get a full instrument definition given the instrument number
	 * @param voiceNumber
	 * @return
	 */
	public InstrumentVoice getInstrumentVoice(int voiceNumber) {
		return(instruments[voiceNumber]);
	}
	
	
	/** 
	 * get the accumulator index, should be zero or one
	 * @return
	 */
	private synchronized Integer getBuildIndex() {
		return(buildIndex);
	}

	
	/**
	 * Used by the Message Receiver, this is the way to increment the count of
	 * an instrument when its associated search string is found
	 * @param instrumentName
	 */
	public void incrementInstrumentCount(String instrumentName) {
		int whichInstrument = instrumentIndexes.get(instrumentName);
		counts[getBuildIndex()][whichInstrument]++;
		log.debug("instrument " + instrumentName + ", " + whichInstrument + " count incred to " +
				counts[getBuildIndex()][whichInstrument]);
	}
	
	private synchronized Integer getPlayIndex() {
		return(buildIndex == 1 ? 0 : 1);
	}

//	/**
//	 * get the counts for the currently-building accumulator
//	 * @return
//	 */
//	public int[] getBuildCounts() {
//		return(counts[getBuildIndex()]);
//	}
	
	
	/**
	 * get the count for a particular instrument out of the right
	 * accumulator
	 * @param voiceNumber
	 * @return
	 */
	public int getPlayCount(int voiceNumber) {
		return(counts[getPlayIndex()][voiceNumber]);
	}

	
	/**
	 * Used by the Sound Builder when it is time to start building the next music
	 * string.  This switches the accumulators so that one is left stable for the
	 * sound builder while the next iteration of counts can be started
	 */
	synchronized void swapIndex() {
		buildIndex = (buildIndex==1) ? 0 : 1;
		for(int x=0; x < MAX_MIDI_VOICES; x++) {
			counts[buildIndex][x] = 0;
		}
	}
}
